import * as dotenv from 'dotenv';

dotenv.config();

const mariadb = require('mariadb');
const prod_pool = mariadb.createPool({
    host: process.env.DB_HOST,
    user: process.env.DB_USER,
    password: process.env.DB_PASSWORD,
    database: process.env.PARTYPLANER_PROD_DATABASE,
    connectionLimit: 5
});
const dev_pool = mariadb.createPool({
    host: process.env.DB_HOST,
    user: process.env.DB_USER,
    password: process.env.DB_PASSWORD,
    database: process.env.PARTYPLANER_DEV_DATABASE,
    connectionLimit: 5
});

/**
 * Used in the getUserData method as return value
 */
export interface UserData {
    username: string;
    email: string;
    firstName: string;
    lastName: string;
    lastLogin: string;
    emailIsVerified: string;
    isPremiumUser: string;
}

/**
 * Used in the getSessionData method as return value
 */
export interface SessionData {
    sessionId: string;
    type: string;
    lastLogin: string;
    lastIp: string;
}

/**
 * Used in the getFriendshipData method as a return value
 */
export interface Friendship {
    friendshipId: string;
    friendId: string;
    friendFirstName: string;
    friendLastName: string;
    friendUsername: string;
}

/**
 * Returns all data about the given user
 * @param useDev If the dev or prod database should be used
 * @param userId The userId of the user to return the data for
 * @return UserData An object containing the user data
 */
export const getUserData = async (useDev: boolean, userId: string): Promise<UserData> => {
    let conn;
    try {
        if (useDev) {
            conn = await dev_pool.getConnection();
        } else {
            conn = await prod_pool.getConnection();
        }

        let rows = await conn.query('SELECT username, email, first_name, last_Name, last_login, email_is_verified, is_premium_user FROM users WHERE user_id = ?', userId);

        let user: UserData = {} as UserData;

        for (let row in rows) {
            if (row !== 'meta') {
                user = {
                    username: rows[row].username,
                    email: rows[row].email,
                    firstName: rows[row].first_name,
                    lastName: rows[row].last_name,
                    lastLogin: rows[row].last_login,
                    emailIsVerified: rows[row].email_is_verified,
                    isPremiumUser: rows[row].is_premium_user
                };
            }
        }

        return user;
    } catch (err) {
        throw err;
    } finally {
        if (conn) {
            conn.end();
        }
    }
};

/**
 * Returns all active sessions of the given user
 * @param useDev If the dev or prod database should be used
 * @param userId The userId of the user to return the sessions for
 * @return SessionData[] A list containing objects with the session data
 */
export const getSessionData = async (useDev: boolean, userId: string): Promise<SessionData[]> => {
    let conn;
    try {
        if (useDev) {
            conn = await dev_pool.getConnection();
        } else {
            conn = await prod_pool.getConnection();
        }

        let rows = await conn.query('SELECT session_id, type, last_login, last_ip FROM sessions WHERE user_id = ? AND valid_until > NOW()', userId);

        let sessions: SessionData[] = [];

        for (let row in rows) {
            if (row !== 'meta') {
                sessions.push({
                    sessionId: rows[row].session_id,
                    type: rows[row].type,
                    lastLogin: rows[row].last_login,
                    lastIp: rows[row].last_ip
                });
            }
        }

        return sessions;
    } catch (err) {
        throw err;
    } finally {
        if (conn) {
            conn.end();
        }
    }
};

/**
 * Returns all friends of the given user
 * @param useDev If the dev or prod database should be used
 * @param userId The userId of the user to fetch the friends for
 * @return Friendship A list of friends
 */
export const getFriendshipData = async (useDev: boolean, userId: string): Promise<Friendship[]> => {
    let conn;
    try {
        if (useDev) {
            conn = await dev_pool.getConnection();
        } else {
            conn = await prod_pool.getConnection();
        }

        let rows = await conn.query('SELECT f.friendship_id, f.friend_id, u.first_name as friend_first_name, u.last_name as friend_last_name, u.username as friend_username FROM friendships f LEFT OUTER JOIN users u ON f.friend_id = u.user_id WHERE f.user_id = ?', userId);

        let friends: Friendship[] = [];

        for (let row in rows) {
            if (row !== 'meta') {
                friends.push({
                    friendshipId: rows[row].friendship_id,
                    friendId: rows[row].friend_id,
                    friendFirstName: rows[row].friend_first_name,
                    friendLastName: rows[row].friend_last_name,
                    friendUsername: rows[row].friend_username
                });
            }
        }

        return friends;
    } catch (err) {
        throw err;
    } finally {
        if (conn) {
            conn.end();
        }
    }
};
